/*****************************************************************************
Copyright(c) 2004 - 2006 Intel Corporation. All rights reserved.

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your option)
any later version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59
Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The full GNU General Public License is included in this distribution in the
file called LICENSE.
*****************************************************************************/
#ifndef NETDMA_H
#define NETDMA_H
#ifdef CONFIG_NET_DMA
#include <linux/dmaengine.h>
#include <linux/skbuff.h>
#include <net/sock.h>
#include <linux/tcp.h>
#include <linux/ipv6.h>

DECLARE_PER_CPU(struct dma_chan *, net_dma);

static inline struct dma_chan *get_softnet_dma(void)
{
	struct dma_chan *chan;
	rcu_read_lock();
	chan = rcu_dereference(__get_cpu_var(net_dma));
	if (chan)
		dma_chan_get(chan);
	rcu_read_unlock();
	return chan;
}

#define IOAT_SOCK_MAGIC 0x494f4154

struct ioat_sock {
	u32			magic;	/* must be first member of ioat_sock */
	struct dma_chan		*dma_chan;
	int			wakeup;
	struct dma_pinned_list	*pinned_list;
	dma_cookie_t		dma_cookie;
	struct sk_buff_head	sk_async_wait_queue;
};

static inline void ioat_sock_init(struct ioat_sock *ioat)
{
	ioat->magic = IOAT_SOCK_MAGIC;
	ioat->dma_chan = NULL;
	ioat->wakeup = 0;
	ioat->pinned_list = NULL;
	ioat->dma_cookie = 0;
	skb_queue_head_init(&ioat->sk_async_wait_queue);
}

struct tcp_ioat_sock {
	struct tcp_sock tcp;
	struct ioat_sock ioat;
};

struct tcp6_ioat_sock {
	struct tcp6_sock tcp6;
	struct ioat_sock ioat;
};

#define __tcp_ioat(sk) (&((struct tcp_ioat_sock *)(sk))->ioat)
static inline struct ioat_sock * tcp_ioat(struct sock *sk)
{
	struct ioat_sock *ioat = __tcp_ioat(sk);
	if (ioat->magic == IOAT_SOCK_MAGIC)
		return ioat;
	return NULL;
}

#define __tcp6_ioat(sk) (&((struct tcp6_ioat_sock *)(sk))->ioat)
static inline struct ioat_sock * tcp6_ioat(struct sock *sk)
{
	struct ioat_sock *ioat = __tcp6_ioat(sk);
	if (ioat->magic == IOAT_SOCK_MAGIC)
		return ioat;
	return NULL;
}

static inline struct ioat_sock * ioat_sk(struct sock *sk)
{
	if (sk->sk_prot->slab_obj_size == sizeof(struct tcp_ioat_sock))
		return tcp_ioat(sk);
	if (sk->sk_prot->slab_obj_size == sizeof(struct tcp6_ioat_sock))
		return tcp6_ioat(sk);
	return NULL;
}

int dma_skb_copy_datagram_iovec(struct dma_chan* chan,
		const struct sk_buff *skb, int offset, struct iovec *to,
		size_t len, struct dma_pinned_list *locked_list);
int dma_async_try_early_copy(struct sock *sk, struct sk_buff *skb, int hlen);

#endif /* CONFIG_NET_DMA */
#endif /* NETDMA_H */
